@@ -27,40 +27,25 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-import cgi
 from datetime import datetime
-try:
-    from email.feedparser import FeedParser
-    from email.header import Header
-except ImportError:
-    from email.Parser import FeedParser
-    from email.Header import Header
 import logging
-import os
 import random
 import re
-from StringIO import StringIO
 import sys
 import traceback
 import unittest
-from urlparse import urlparse
 
+import httplib2
 import mox
-from oauth.oauth import OAuthConsumer, OAuthToken
 import simplejson as json
 
 import typepad
 from tests import utils
 
 
-def json_equals(data, text):
-    otherdata = json.loads(text)
-    return bool(data == otherdata)
-
-
 def json_equals_func(data):
     def confirm_equals_data(text):
-        return json_equals(data, text)
+        return utils.json_equals(data, text)
     return confirm_equals_data
 
 
@@ -90,8 +75,8 @@ requests = {
                     'types': ('tag:api.typepad.com,2009:Admin',
                               'tag:api.typepad.com,2009:Member'),
                 },
-                'target': {"objectTypes": ["tag:api.typepad.com,2009:Group"]},
-                'source': {"displayName": "Mike", "objectTypes": ["tag:api.typepad.com,2009:User"]},
+                'target': {"objectType": "Group"},
+                'source': {"displayName": "Mike", "objectType": "User"},
             }] + [{
                 'status': {
                     'created': {
@@ -99,15 +84,15 @@ requests = {
                     },
                     'types': ('tag:api.typepad.com,2009:Member',),
                 },
-                'target': {"objectTypes": ["tag:api.typepad.com,2009:Group"]},
+                'target': {"objectType": "Group"},
                 'source': u,
             } for u in (
-                {"displayName": "Sherry Monaco", "objectTypes": ["tag:api.typepad.com,2009:User"]},
-                {"displayName": "Francesca Coppola", "objectTypes": ["tag:api.typepad.com,2009:User"]},
-                {"displayName": "David Rosato", "objectTypes": ["tag:api.typepad.com,2009:User"]},
-                {"displayName": "Edgar Bach", "objectTypes": ["tag:api.typepad.com,2009:User"]},
-                {"displayName": "Jarad Mccaw", "objectTypes": ["tag:api.typepad.com,2009:User"]},
-                {"displayName": "Deanna Conti", "objectTypes": ["tag:api.typepad.com,2009:User"]},
+                {"displayName": "Sherry Monaco", "objectType": "User"},
+                {"displayName": "Francesca Coppola", "objectType": "User"},
+                {"displayName": "David Rosato", "objectType": "User"},
+                {"displayName": "Edgar Bach", "objectType": "User"},
+                {"displayName": "Jarad Mccaw", "objectType": "User"},
+                {"displayName": "Deanna Conti", "objectType": "User"},
             )],
         }),
     ),
@@ -122,15 +107,15 @@ requests = {
               'accept': 'application/json',
               'content-type': 'application/json',
           },
-          'body': """{"content": "Hi this post has some content is it not nifty", "objectTypes": ["tag:api.typepad.com,2009:Post"], "categories": [{"term": "fred"}, {"term": "wilma"}], "title": "New post #47"}""",
+          'body': """{"content": "Hi this post has some content is it not nifty", "objectType": "Post", "categories": ["fred", "wilma"], "title": "New post #47"}""",
           'method': 'POST' },
         { 'status': 201,
           'location': 'http://127.0.0.1:8000/assets/307.json',
           'content': """{
               "title": "New post #47",
               "content": "Hi this post has some content is it not nifty",
-              "objectTypes": ["tag:api.typepad.com,2009:Post"],
-              "categories": [{"term": "fred"}, {"term": "wilma"}],
+              "objectType": "Post",
+              "categories": ["fred", "wilma"],
               "published": "2009-03-23T00:00:00Z",
               "updated": "2009-03-23T00:00:00Z"
           }""" },
@@ -141,8 +126,8 @@ requests = {
         """{
             "title": "New post #47",
             "content": "Hi this post has some content is it not nifty",
-            "objectTypes": ["tag:api.typepad.com,2009:Post"],
-            "categories": [{"term": "fred"}, {"term": "wilma"}],
+            "objectType": "Post",
+            "categories": ["fred", "wilma"],
             "published": "2009-03-23T00:00:00Z",
             "updated": "2009-03-23T00:00:00Z"
         }""",
@@ -164,7 +149,7 @@ requests = {
         """{
             "title": "Fames Vivamus Placerat at Condimentum at Primis Consectetuer Nonummy Inceptos Porta dis",
             "content": "Posuere felis vestibulum nibh justo vitae elementum.",
-            "objectTypes": ["tag:api.typepad.com,2009:Post"]
+            "objectType": "Post"
         }""",
     ),
     'put_mutated_post': (
@@ -174,21 +159,31 @@ requests = {
             'accept': 'application/json',
             'content-type': 'application/json',
           },
-          'body': mox.Func(json_equals_func({"content": "Yay this is my post", "objectTypes": ["tag:api.typepad.com,2009:Post"], "title": "Omg hai"})),
+          'body': mox.Func(json_equals_func({"content": "Yay this is my post", "objectType": "Post", "title": "Omg hai"})),
           'method': 'PUT' },
-        { 'content': """{"content": "Yay this is my post", "objectTypes": ["tag:api.typepad.com,2009:Post"], "title": "Omg hai"}""",
+        { 'content': """{"content": "Yay this is my post", "objectType": "Post", "title": "Omg hai"}""",
           'etag': 'xyz' },
     ),
+    'post_comment_preview': (
+        { 'uri': 'http://127.0.0.1:8000/assets/1/make-comment-preview.json',
+          'headers': {
+            'accept': 'application/json',
+            'content-type': 'application/json',
+          },
+          'body': mox.Func(json_equals_func({"content": "hi hello my comment hi"})),
+          'method': 'POST' },
+        """{"comment": {"content": "hi hello my comment hi"}}""",
+    ),
 }
 
 
-class TestAsset(unittest.TestCase):
+class TestHttp(unittest.TestCase):
 
     def http(self, key, credentials=None):
         try:
             req = requests[key]
         except KeyError:
-            raise Exception('No such mock request %s' % (callername,))
+            raise Exception('No such mock request %s' % key)
 
         mock = utils.mock_http(*req)
         mock.endpoint = 'http://api.typepad.com'
@@ -250,7 +245,7 @@ class TestAsset(unittest.TestCase):
         p = typepad.Post(
             title="New post #47",
             content="Hi this post has some content is it not nifty",
-            categories=[typepad.Tag(term="fred", count=None), typepad.Tag(term="wilma", count=None)],
+            categories=["fred", "wilma"],
         )
 
         h = self.http('create_post', credentials=('mmalone@example.com', 'password'))
@@ -263,8 +258,7 @@ class TestAsset(unittest.TestCase):
         self.assert_(p.published is not None)
 
         self.assert_(len(p.categories), 2)
-        tags = sorted(p.categories, key=lambda x: x.term)
-        self.assertEquals(tags, [typepad.Tag(term="fred", count=None), typepad.Tag(term="wilma", count=None)])
+        self.assertEquals(p.categories, ["fred", "wilma"])
 
         h = self.http('get_created_post')
         post_got = typepad.Post.get(p._location, http=h)
@@ -305,33 +299,73 @@ class TestAsset(unittest.TestCase):
         self.assertEquals(e.title, 'Omg hai')
         self.assertNotEqual(e._etag, old_etag)
 
+    def test_action_endpoint(self):
+        asset = typepad.Asset.get('http://127.0.0.1:8000/assets/1.json')
 
-class TestLocally(unittest.TestCase):
+        h = self.http('post_comment_preview')
+        preview = asset.make_comment_preview(content='hi hello my comment hi')
+        mox.Verify(h)
+
+
+class TestUrlId(unittest.TestCase):
+
+    def check_url_id(self, cls):
+        works = (
+            lambda: cls.get_by_url_id('6a00d83451ce6b69e20120a81fb3a4970c'),
+            lambda: cls.get_by_id('tag:api.typepad.com,2009:6a00d83451ce6b69e20120a81fb3a4970c'),
+            lambda: cls.get_by_id('6a00d83451ce6b69e20120a81fb3a4970c'),
+        )
+
+        for work in works:
+            obj = work()
+            self.assert_('6a00d83451ce6b69e20120a81fb3a4970c' in obj._location)
+            self.assertEqual(obj.url_id, '6a00d83451ce6b69e20120a81fb3a4970c')  # doesn't deliver
+            self.assertEqual(obj.xid, '6a00d83451ce6b69e20120a81fb3a4970c')
+            self.assertEqual(obj.id, 'tag:api.typepad.com,2009:6a00d83451ce6b69e20120a81fb3a4970c')
+
+        self.assertRaises(ValueError, lambda: cls.get_by_url_id(''))
+        self.assertRaises(ValueError, lambda: cls.get_by_id(''))
+
+        # See if link generation with make_self_link() works.
+        obj = cls()
+        obj.update_from_dict({'urlId': '6a00d83451ce6b69e20120a81fb3a4970c'})
+        self.assert_(obj._location is not None)
+        # UserProfile's doesn't end in /urlId.json, so just check that it's in there somewhere.
+        self.assert_('6a00d83451ce6b69e20120a81fb3a4970c' in obj._location)
+
+    def _check(cls):
+        def check_for_class(self):
+            self.check_url_id(cls)
+        check_for_class.__name__ = 'test_%s' % cls.__name__.lower()
+        return check_for_class
+
+    test_asset = _check(typepad.Asset)
+    test_blog = _check(typepad.Blog)
+    test_event = _check(typepad.Event)
+    test_externalfeedsubscription = _check(typepad.ExternalFeedSubscription)
+    test_favorite = _check(typepad.Favorite)
+    test_relationship = _check(typepad.Relationship)
+    test_group = _check(typepad.Group)
+    test_user = _check(typepad.User)
+    test_userprofile = _check(typepad.UserProfile)
+
+
+class TestAsset(unittest.TestCase):
 
     def check_classy_assets(self, make_asset):
         data = {
             'id':          '7',
             'urlId':       '7',
             'title':       'some asset',
-            'objectTypes': ('http://example.com/internet/',),
+            'objectType':  'internet',
         }
 
         # Unknown objectTypes should yield an Asset.
         a = make_asset(data)
         self.assertEquals(type(a), typepad.Asset)
 
-        # Known objectTypes should yield that type.
-        data['objectTypes'] = ('tag:api.typepad.com,2009:Post',)
-        a = make_asset(data)
-        self.assertEquals(type(a), typepad.Post)
-
-        # Type name needn't match the word in objectTypes.
-        data['objectTypes'] = ('tag:api.typepad.com,2009:Link',)
-        a = make_asset(data)
-        self.assertEquals(type(a), typepad.LinkAsset)
-
-        # Ignore extra objectTypes.
-        data['objectTypes'] = ('http://example.com/internet/', 'tag:api.typepad.com,2009:Post')
+        # A known objectType should yield that type.
+        data['objectType'] = 'Post'
         a = make_asset(data)
         self.assertEquals(type(a), typepad.Post)
 
@@ -353,7 +387,7 @@ class TestLocally(unittest.TestCase):
             'id':          '7',
             'urlId':       '7',
             'title':       'some asset',
-            'objectTypes': ('http://example.com/internet/',),
+            'objectType':  'internet',
         }
 
         class AssetRefKeeper(typepad.TypePadObject):
@@ -364,7 +398,7 @@ class TestLocally(unittest.TestCase):
         ar = AssetRefKeeper.from_dict({'assetref': data}).assetref
         self.assert_(isinstance(ar, typepad.AssetRef))
 
-        data['objectTypes'] = ('tag:api.typepad.com,2009:Post',)
+        data['objectType'] = 'Post'
 
         # Even with a known object type, AssetRefs stay AssetRefs.
         ar = typepad.AssetRef.from_dict(data)
@@ -372,6 +406,51 @@ class TestLocally(unittest.TestCase):
         ar = AssetRefKeeper.from_dict({'assetref': data}).assetref
         self.assert_(isinstance(ar, typepad.AssetRef))
 
+    def test_asset_ref(self):
+        asset = typepad.Asset(
+            id='tag:api.typepad.com,2009:6a00d83451ce6b69e20120a81fb3a4970c',
+            url_id='6a00d83451ce6b69e20120a81fb3a4970c',
+            author=typepad.User(
+                url_id='6p00d83451ce6b69e2',
+            ),
+            object_types=['tag:api.typepad.com,2009:Post'],
+            object_type='Post'
+        )
+
+        ref = asset.asset_ref
+
+        self.assert_(isinstance(ref, typepad.AssetRef))
+        self.assertEqual(ref.id, 'tag:api.typepad.com,2009:6a00d83451ce6b69e20120a81fb3a4970c')
+        self.assertEqual(ref.url_id, '6a00d83451ce6b69e20120a81fb3a4970c')
+        self.assert_(ref.author is not None)
+        self.assert_(isinstance(ref.author, typepad.User))
+        self.assertEqual(ref.author.url_id, '6p00d83451ce6b69e2')
+        self.assertEqual(ref.object_type, 'Post')
+        self.assertEqual(ref.object_types, ['tag:api.typepad.com,2009:Post'])
+
+    def test_stringification(self):
+        asset = typepad.Asset(
+            title='this is my title',
+            content='This is my content.',
+        )
+        self.assertEqual(str(asset), 'this is my title')
+        self.assertEqual(unicode(asset), 'this is my title')
+
+        asset = typepad.Asset(
+            content='This is my content.',
+        )
+        self.assertEqual(str(asset), 'This is my content.')
+        self.assertEqual(unicode(asset), 'This is my content.')
+
+        asset = typepad.Asset(
+            title=u'I\xc3\xb1t\xc3\xabrn\xc3\xa2ti\xc3\xb4n\xc3\xa0liz\xc3\xa6ti\xc3\xb8n',
+        )
+        self.assertRaises(UnicodeError, lambda: str(asset))
+        self.assertEqual(unicode(asset), u'I\xc3\xb1t\xc3\xabrn\xc3\xa2ti\xc3\xb4n\xc3\xa0liz\xc3\xa6ti\xc3\xb8n')
+
+
+class TestRelationship(unittest.TestCase):
+
     def test_group_membership_with_timestamps(self):
         data = {
             'created': {
@@ -402,15 +481,15 @@ class TestLocally(unittest.TestCase):
         self.assertEquals(types[1], 'tag:api.typepad.com,2009:Member')
         self.assertEquals(created[types[1]], datetime(2009, 1, 3, 0, 0, 0))
 
-    def test_relationships(self):
+    def test_object_typing(self):
         data = {
             'source': {
                 'displayName': 'Nikola',
-                'objectTypes': ['tag:api.typepad.com,2009:User'],
+                'objectType':  'User',
             },
             'target': {
                 'displayName': 'Theophila',
-                'objectTypes': ['tag:api.typepad.com,2009:User'],
+                'objectType':  'User',
             },
             'status': {
                 'types': ['tag:api.typepad.com,2009:Contact'],
@@ -427,7 +506,7 @@ class TestLocally(unittest.TestCase):
         self.assert_(isinstance(r.target, typepad.User))
 
         data['status']['types']       = ['tag:api.typepad.com,2009:Member']
-        data['source']['objectTypes'] = ['tag:api.typepad.com,2009:Group']
+        data['source']['objectType']  = 'Group'
         r = typepad.Relationship.from_dict(data)
         self.assert_(isinstance(r.source, typepad.Group))
         self.assert_(isinstance(r.target, typepad.User))
@@ -447,252 +526,141 @@ class TestLocally(unittest.TestCase):
         self.assert_(isinstance(r.source, typepad.Group))
         self.assert_(isinstance(r.target, typepad.User))
 
-    def test_user_url_id(self):
-        def valid(v):
-            self.assert_(typepad.User.get_by_url_id(v))
-
-        def invalid(i):
-            self.assertRaises(ValueError,
-                              lambda: typepad.User.get_by_url_id(i))
-
-        valid('asfdasf')
-        valid('OHHAI')
-        valid('thx1138')
-        valid('url_id')
-        valid('1138')
-        valid('____')
-        valid('_')
-        valid('7')
-
-        invalid('url id')
-        invalid('url-id')
-        invalid('')
-        invalid('Why?')
-        invalid('^hat^hat^')
-
-
-class TestBrowserUpload(unittest.TestCase):
-
-    def setUp(self):
-        self.typepad_client = typepad.client
-
-    def tearDown(self):
-        typepad.client = self.typepad_client
-        del self.typepad_client
-
-        for x in ('headers', 'body'):
-            try:
-                delattr(self, x)
-            except AttributeError:
-                pass
-
-    def saver(self, fld):
-        def save_data(data):
-            setattr(self, fld, data)
-            return True
-        return save_data
-
-    def message_from_response(self, headers, body):
-        fp = FeedParser()
-        for header, value in headers.iteritems():
-            fp.feed("%s: %s\n" % (header, Header(value).encode()))
-        fp.feed("\n")
-        fp.feed(body)
-        response = fp.close()
-        return response
-
-    def test_basic(self):
-        request = {
-            'uri': mox.Func(self.saver('uri')),
-            'method': 'POST',
-            'headers': mox.Func(self.saver('headers')),
-            'body': mox.Func(self.saver('body')),
-        }
-        response = {
-            'status': 201,  # created
+    def test_type_checking(self):
+        data = {
+            'source': {
+                'displayName': 'Nikola',
+                'objectType':  'User',
+            },
+            'target': {
+                'displayName': 'Theophila',
+                'objectType':  'User',
+            },
+            'status': {
+                'types': ['tag:api.typepad.com,2009:Contact'],
+            },
         }
 
-        http = typepad.TypePadClient()
-        typepad.client = http
-        http.add_credentials(
-            OAuthConsumer('consumertoken', 'consumersecret'),
-            OAuthToken('tokentoken', 'tokensecret'),
-            domain='api.typepad.com',
-        )
+        r = typepad.Relationship.from_dict(data)
+        self.assert_(isinstance(r.source, typepad.User))
+        self.assert_(isinstance(r.target, typepad.User))
 
-        mock = mox.Mox()
-        mock.StubOutWithMock(http, 'request')
-        http.request(**request).AndReturn((response, ''))
-        mock.ReplayAll()
+        self.assert_(not r.is_member())
+        self.assert_(not r.is_admin())
+        self.assert_(not r.is_blocked())
 
-        asset = typepad.Photo()
-        asset.title = "Fake photo"
-        asset.content = "This is a made-up photo for testing automated browser style upload."
+        data['source']['objectType'] = 'Group'
+        data['status']['types'] = ['tag:api.typepad.com,2009:Member']
+        r = typepad.Relationship.from_dict(data)
 
-        fileobj = StringIO('hi hello pretend file')
-        brupload = typepad.BrowserUploadEndpoint()
-        brupload.upload(asset, fileobj, "image/png",
-            post_type='photo')
+        self.assert_(r.is_member())
+        self.assert_(not r.is_admin())
+        self.assert_(not r.is_blocked())
 
-        mock.VerifyAll()
+        data['status']['types'] = ['tag:api.typepad.com,2009:Member', 'tag:api.typepad.com,2009:Asfdasf',
+            'tag:api.typepad.com,2009:Admin']
+        r = typepad.Relationship.from_dict(data)
 
-        self.assert_(self.uri)
-        self.assert_(self.uri.startswith('http://api.typepad.com/browser-upload.json'))
-        uriparts = list(urlparse(self.uri))
-        querydict = cgi.parse_qs(uriparts[4])
+        self.assert_(r.is_member())
+        self.assert_(r.is_admin())
+        self.assert_(not r.is_blocked())
 
-        # TODO: really verify the signature
+        data['status']['types'] = ['tag:api.typepad.com,2009:Blocked']
+        r = typepad.Relationship.from_dict(data)
 
-        # Verify the headers and body.
-        self.assert_(self.headers)
-        self.assert_(self.body)
-        responsemsg = self.message_from_response(self.headers, self.body)
+        self.assert_(not r.is_member())
+        self.assert_(not r.is_admin())
+        self.assert_(r.is_blocked())
 
-        content_type = responsemsg.get_content_type()
-        self.assert_(content_type)
-        self.assert_(not responsemsg.defects)
+    def test_block(self):
+        real_typepad_client = typepad.client
+        typepad.client = mox.MockObject(httplib2.Http)
+        try:
+            resp = httplib2.Response({
+                'status':           200,
+                'etag':             '7',
+                'content-type':     'application/json',
+                'content-location': 'http://127.0.0.1:8000/relationships/6r00d83451ce6b69e20120a81fb3a4970c/status.json',
+            })
+            typepad.client.request(
+                uri='http://127.0.0.1:8000/relationships/6r00d83451ce6b69e20120a81fb3a4970c/status.json',
+                headers={'accept': 'application/json'},
+            ).AndReturn((resp, """{"types": ["tag:api.typepad.com,2009:Member"]}"""))
+            resp = httplib2.Response({
+                'status':           200,
+                'etag':             '9',
+                'content-type':     'application/json',
+                'content-location': 'http://127.0.0.1:8000/relationships/6r00d83451ce6b69e20120a81fb3a4970c/status.json',
+            })
+            typepad.client.request(
+                uri='http://127.0.0.1:8000/relationships/6r00d83451ce6b69e20120a81fb3a4970c/status.json',
+                method='PUT',
+                headers={'if-match': '7', 'accept': 'application/json', 'content-type': 'application/json'},
+                body=mox.Func(json_equals_func({"types": ["tag:api.typepad.com,2009:Blocked"]})),
+            ).AndReturn((resp, """{"types": ["tag:api.typepad.com,2009:Blocked"]}"""))
+            mox.Replay(typepad.client)
 
-        # Check that the unparsed body has its internal mime headers
-        # separated by full CRLFs, not just LFs.
-        self.assert_('\r\nContent-Type:' in self.body)
-        # Check that boundaries are separated by full CRLFs too.
-        boundary = responsemsg.get_param('boundary')
-        self.assert_(boundary + '\r\n' in self.body)
+            r = typepad.Relationship.get('http://127.0.0.1:8000/relationships/6r00d83451ce6b69e20120a81fb3a4970c.json')
+            r.block()
 
-        # Make sure we're only putting credentials in the query string, not
-        # the headers.
-        self.assert_('oauth_signature' in querydict)
-        self.assert_('authorization' not in responsemsg)
-        self.assert_('Authorization' not in responsemsg)
+            mox.Verify(typepad.client)
 
-        bodyparts = responsemsg.get_payload()
-        self.assertEquals(len(bodyparts), 3)
-        bodyparts = dict((part.get_param('name', header='content-disposition'),
-            part) for part in bodyparts)
+        finally:
+            typepad.client = real_typepad_client
 
-        self.assertEquals(bodyparts['post_type'].get_payload(), 'photo')
-        self.assert_('redirect_to' not in bodyparts)
+    def test_leave(self):
+        real_typepad_client = typepad.client
+        typepad.client = mox.MockObject(httplib2.Http)
+        try:
+            resp = httplib2.Response({
+                'status':           200,
+                'etag':             '7',
+                'content-type':     'application/json',
+                'content-location': 'http://127.0.0.1:8000/relationships/6r00d83451ce6b69e20120a81fb3a4970c/status.json',
+            })
+            typepad.client.request(
+                uri='http://127.0.0.1:8000/relationships/6r00d83451ce6b69e20120a81fb3a4970c/status.json',
+                headers={'accept': 'application/json'},
+            ).AndReturn((resp, """{"types": ["tag:api.typepad.com,2009:Member"]}"""))
+            resp = httplib2.Response({
+                'status':           200,
+                'etag':             '9',
+                'content-type':     'application/json',
+                'content-location': 'http://127.0.0.1:8000/relationships/6r00d83451ce6b69e20120a81fb3a4970c/status.json',
+            })
+            typepad.client.request(
+                uri='http://127.0.0.1:8000/relationships/6r00d83451ce6b69e20120a81fb3a4970c/status.json',
+                method='PUT',
+                headers={'if-match': '7', 'accept': 'application/json', 'content-type': 'application/json'},
+                body=mox.Func(json_equals_func({"types": []})),
+            ).AndReturn((resp, """{"types": []}"""))
+            mox.Replay(typepad.client)
 
-        asset_json = bodyparts['asset'].get_payload()
-        self.assert_(json_equals({
-            'title': 'Fake photo',
-            'content': 'This is a made-up photo for testing automated browser style upload.',
-            'objectTypes': ['tag:api.typepad.com,2009:Photo'],
-        }, asset_json))
+            r = typepad.Relationship.get('http://127.0.0.1:8000/relationships/6r00d83451ce6b69e20120a81fb3a4970c.json')
+            r.leave()
 
-        filepart = bodyparts['file']
-        self.assertEquals(filepart.get_payload(decode=False), 'hi hello pretend file')
-        filelength = filepart.get('content-length')
-        self.assertEquals(int(filelength), len('hi hello pretend file'))
-        filename = filepart.get_param('filename', header='content-disposition')
-        self.assert_(filename)
+            mox.Verify(typepad.client)
 
-    def test_redirect(self):
-        request = {
-            'uri': mox.Func(self.saver('uri')),
-            'method': 'POST',
-            'headers': mox.Func(self.saver('headers')),
-            'body': mox.Func(self.saver('body')),
-        }
-        response = {
-            'status': 302,
-            'location': 'http://client.example.com/hi',
-        }
+        finally:
+            typepad.client = real_typepad_client
 
-        http = typepad.TypePadClient()
-        typepad.client = http
-        http.add_credentials(
-            OAuthConsumer('consumertoken', 'consumersecret'),
-            OAuthToken('tokentoken', 'tokensecret'),
-            domain='api.typepad.com',
-        )
 
-        mock = mox.Mox()
-        mock.StubOutWithMock(http, 'request')
-        http.request(**request).AndReturn((response, ''))
-        mock.ReplayAll()
+class TestUserAndUserProfile(unittest.TestCase):
 
-        asset = typepad.Photo()
-        asset.title = "Fake photo"
-        asset.content = "This is a made-up photo for testing automated browser style upload."
+    def test_get_self(self):
+        me = typepad.User.get_self()
+        self.assert_(me._location.endswith('/users/@self.json'))
 
-        fileobj = StringIO('hi hello pretend file')
-        brupload = typepad.BrowserUploadEndpoint()
-        brupload.upload(asset, fileobj, "image/png",
-            redirect_to='http://client.example.com/hi',
-            post_type='photo')
+    def test_related_profile(self):
+        my_profile = typepad.UserProfile.get_by_url_id('6p00d83451ce6b69e2')
+        me = my_profile.user
+        self.assert_(isinstance(me, typepad.User))
+        self.assertEqual(me.url_id, '6p00d83451ce6b69e2')
 
-        mock.VerifyAll()
-
-        # Verify the headers and body.
-        self.assert_(self.headers)
-        self.assert_(self.body)
-        response = self.message_from_response(self.headers, self.body)
-
-        bodyparts = response.get_payload()
-        self.assertEquals(len(bodyparts), 4)
-        bodyparts = dict((part.get_param('name', header='content-disposition'),
-            part) for part in bodyparts)
-
-        # Confirm that the redirect_to was sent.
-        self.assert_('redirect_to' in bodyparts)
-        self.assertEquals(bodyparts['redirect_to'].get_payload(),
-            'http://client.example.com/hi')
-
-    def test_real_file(self):
-        request = {
-            'uri': mox.Func(self.saver('uri')),
-            'method': 'POST',
-            'headers': mox.Func(self.saver('headers')),
-            'body': mox.Func(self.saver('body')),
-        }
-        response = {
-            'status': 201,
-        }
-
-        http = typepad.TypePadClient()
-        typepad.client = http
-        http.add_credentials(
-            OAuthConsumer('consumertoken', 'consumersecret'),
-            OAuthToken('tokentoken', 'tokensecret'),
-            domain='api.typepad.com',
-        )
-
-        mock = mox.Mox()
-        mock.StubOutWithMock(http, 'request')
-        http.request(**request).AndReturn((response, ''))
-        mock.ReplayAll()
-
-        asset = typepad.Photo()
-        asset.title = "One-by-one png"
-        asset.content = "This is a 1&times;1 transparent PNG."
-
-        fileobj = file(os.path.join(os.path.dirname(__file__), 'onebyone.png'))
-        brupload = typepad.BrowserUploadEndpoint()
-        brupload.upload(asset, fileobj, "image/png",
-            post_type='photo')
-
-        mock.VerifyAll()
-
-        response = self.message_from_response(self.headers, self.body)
-
-        (filepart,) = [part for part in response.get_payload()
-            if part.get_param('name', header='content-disposition') == 'file']
-
-        self.assertEquals(filepart.get_content_type(), 'image/png')
-
-        # If there's a transfer encoding, it has to be the identity encoding.
-        transenc = filepart.get('content-transfer-encoding')
-        self.assert_(transenc is None or transenc == 'identity')
-
-        # Check that the file content is equivalent without decoding.
-        fileobj.seek(0)
-        filecontent = fileobj.read()
-        fileobj.close()
-        self.assertEquals(filepart.get_payload(decode=False), filecontent)
-
-        filelength = filepart.get('content-length')
-        self.assertEquals(int(filelength), len(filecontent))
+        me = typepad.User.get_by_url_id('6p00d83451ce6b69e2')
+        my_profile = me.profile
+        self.assert_(isinstance(my_profile, typepad.UserProfile))
+        self.assertEqual(my_profile.url_id, '6p00d83451ce6b69e2')
 
 
 if __name__ == '__main__':
