@@ -28,7 +28,6 @@
 # POSSIBILITY OF SUCH DAMAGE.
 
 """
-Tests drawn from https://intranet.sixapart.com/wiki/index.php/TPX:API_Endpoints_Implemented_for_Potion
 
     # TODO
     # - featured members (i don't think we have filters for them yet)
@@ -44,7 +43,7 @@ Input test data should have this form:
             'backend_url': '...',
             'oauth_consumer_key': '...',
             'oauth_consumer_secret': '...',
-            'cookies': { 'micro_pool_id': '1' }
+            'cookies': { 'cookie_name': 'value' }
         },
         'admin': {
             'xid': '...',
@@ -98,6 +97,8 @@ is through the test method name, as tests are run in alphabetic order.
     5 - POST requests for parent objects (assets)
     6 - POST requests for child objects (favorites, comments)
 
+    6z - HEAD, OPTIONS requests made after POSTs but before DELETEs
+
     7 - DELETE requests for child objects (favorites, comments)
     8 - DELETE requests for top-level objects (assets)
 
@@ -106,27 +107,27 @@ is through the test method name, as tests are run in alphabetic order.
 """
 
 
+import base64
+import cgi
 import httplib
 import logging
 import os
 from pprint import pprint
 import re
+from StringIO import StringIO
 import time
 import unittest
 from urllib import urlencode, unquote
 from urlparse import urlsplit, urlunsplit, urlparse
-import cgi
-import base64
-from StringIO import StringIO
 
 import httplib2
 import nose
 import nose.plugins.attrib
 import nose.tools
 from oauth import oauth
+from remoteobjects.http import HttpObject
 import simplejson as json
 
-from remoteobjects.http import HttpObject
 import typepad
 
 
@@ -147,7 +148,7 @@ def setUpModule():
     testdata['assets'] = []
     testdata['assets_created'] = []
     testdata['comments_created'] = []
-    if 'coookies' in testdata['configuration']:
+    if 'cookies' in testdata['configuration']:
         typepad.client.cookies.update(testdata['configuration']['cookies'])
 
 
@@ -164,7 +165,7 @@ def attr(*args, **kwargs):
     def wrap(fn):
         user = kwargs.get('user', None)
         if fn.__doc__ is not None:
-            m = re.match(r'^(GET|DELETE|PUT|POST) ', fn.__doc__)
+            m = re.match(r'^(GET|DELETE|PUT|POST|OPTIONS|HEAD) ', fn.__doc__)
             if m is not None:
                 verb = m.groups()[0]
                 kwargs['method'] = verb
@@ -193,7 +194,7 @@ class TestTypePad(unittest.TestCase):
         typepad.Group.get_by_url_id(group_id).post_assets.post(post1)
         self.assertValidAsset(post1)
 
-        self.testdata['assets_created'].append(post1.xid)
+        self.testdata['assets_created'].append(post1.url_id)
 
         # lets pause to reflect (and allow timestamps between these assets to differ)
         time.sleep(2)
@@ -203,7 +204,7 @@ class TestTypePad(unittest.TestCase):
         typepad.Group.get_by_url_id(group_id).post_assets.post(post2)
         self.assertValidAsset(post2)
 
-        self.testdata['assets_created'].append(post2.xid)
+        self.testdata['assets_created'].append(post2.url_id)
 
         # the older post should be favorited
 
@@ -240,7 +241,7 @@ class TestTypePad(unittest.TestCase):
         self.assertEquals(comment1.in_reply_to.url_id, post1.url_id)
         self.assertEquals(comment1.content, content)
 
-        self.testdata['comments_created'].append(comment1.xid)
+        self.testdata['comments_created'].append(comment1.url_id)
 
         content = 'Yeah, what he said.'
 
@@ -252,7 +253,7 @@ class TestTypePad(unittest.TestCase):
         self.assertValidAsset(comment2)
         self.assertEquals(comment2.in_reply_to.url_id, post1.url_id)
 
-        self.testdata['comments_created'].append(comment2.xid)
+        self.testdata['comments_created'].append(comment2.url_id)
 
         # now, load testdata['assets'] with these test assets
         self.load_test_assets()
@@ -336,6 +337,40 @@ class TestTypePad(unittest.TestCase):
         asset = typepad.Asset.get_by_url_id('invalid')
         self.assertNotFound(typepad.client.complete_batch)
 
+    @attr(user='group')
+    def test_8a_OPTIONS_assets_id__post__by_group(self):
+        """OPTIONS /assets/<id>.json (group)
+
+        Tests OPTIONS of an asset using group credentials.
+        """
+
+        self.assert_(len(self.testdata['assets_created']))
+
+        asset_id = self.testdata['assets_created'][0]
+
+        typepad.client.batch_request()
+        opt = typepad.Asset.get_by_url_id(asset_id).options()
+        typepad.client.complete_batch()
+
+        self.assert_(not opt.can_delete(), "group should NOT be able to delete")
+
+    @attr(user='group')
+    def test_8a_OPTIONS_assets_id__comment__by_group(self):
+        """OPTIONS /assets/<id>.json (group)
+
+        Tests OPTIONS of a comment using group credentials.
+        """
+
+        self.assert_(len(self.testdata['comments_created']))
+
+        asset_id = self.testdata['comments_created'][0]
+
+        typepad.client.batch_request()
+        opt = typepad.Asset.get_by_url_id(asset_id).options()
+        typepad.client.complete_batch()
+
+        self.assert_(not opt.can_delete(), "group should NOT be able to delete comment")
+
     @attr(user='group')
     def test_8_DELETE_assets_id__by_group(self):
         """DELETE /assets/<id>.json (group)
@@ -374,6 +409,40 @@ class TestTypePad(unittest.TestCase):
         self.assertValidAsset(asset)
         self.assertForbidden(asset.delete)
 
+    @attr(user='admin')
+    def test_8a_OPTIONS_assets_id__post__by_admin(self):
+        """OPTIONS /assets/<id>.json (admin)
+
+        Tests OPTIONS of an asset using admin credentials.
+        """
+
+        self.assert_(len(self.testdata['assets_created']))
+
+        asset_id = self.testdata['assets_created'][0]
+
+        typepad.client.batch_request()
+        opt = typepad.Asset.get_by_url_id(asset_id).options()
+        typepad.client.complete_batch()
+
+        self.assert_(opt.can_delete(), "admin should be able to delete")
+
+    @attr(user='admin')
+    def test_8a_OPTIONS_assets_id__comment__by_admin(self):
+        """OPTIONS /assets/<id>.json (admin)
+
+        Tests OPTIONS of an comment using admin credentials.
+        """
+
+        self.assert_(len(self.testdata['comments_created']))
+
+        asset_id = self.testdata['comments_created'][0]
+
+        typepad.client.batch_request()
+        opt = typepad.Asset.get_by_url_id(asset_id).options()
+        typepad.client.complete_batch()
+
+        self.assert_(opt.can_delete(), "admin should be able to delete comment")
+
     @attr(user='admin')
     def test_8_DELETE_assets_id__by_admin(self):
         """DELETE /assets/<id>.json (admin)
@@ -381,9 +450,6 @@ class TestTypePad(unittest.TestCase):
         Tests deletion of an asset using admin credentials.
         """
 
-        raise nose.SkipTest(
-            'FIXME: https://intranet.sixapart.com/bugs/default.asp?87922')
-
         self.assert_(len(self.testdata['assets_created']))
 
         asset_id = self.testdata['assets_created'].pop()
@@ -407,9 +473,6 @@ class TestTypePad(unittest.TestCase):
         Tests deletion of a comment using admin credentials.
         """
 
-        raise nose.SkipTest(
-            'FIXME: https://intranet.sixapart.com/bugs/default.asp?87922')
-
         self.assert_(len(self.testdata['comments_created']))
 
         asset_id = self.testdata['comments_created'].pop()
@@ -426,6 +489,40 @@ class TestTypePad(unittest.TestCase):
         asset = typepad.Asset.get_by_url_id(asset_id)
         self.assertNotFound(typepad.client.complete_batch)
 
+    @attr(user='member')
+    def test_6z_OPTIONS_assets_id__post__by_member(self):
+        """OPTIONS /assets/<id>.json (member)
+
+        Tests OPTIONS of an asset using member credentials.
+        """
+
+        self.assert_(len(self.testdata['assets_created']))
+
+        asset_id = self.testdata['assets_created'][0]
+
+        typepad.client.batch_request()
+        opt = typepad.Asset.get_by_url_id(asset_id).options()
+        typepad.client.complete_batch()
+
+        self.assert_(opt.can_delete(), "member should be able to delete")
+
+    @attr(user='member')
+    def test_6z_OPTIONS_assets_id__comment__by_member(self):
+        """OPTIONS /assets/<id>.json (member)
+
+        Tests OPTIONS of an asset using member credentials.
+        """
+
+        self.assert_(len(self.testdata['comments_created']))
+
+        asset_id = self.testdata['comments_created'][0]
+
+        typepad.client.batch_request()
+        opt = typepad.Asset.get_by_url_id(asset_id).options()
+        typepad.client.complete_batch()
+
+        self.assert_(opt.can_delete(), "member should be able to delete comment")
+
     @attr(user='member')
     def test_9_DELETE_assets_id__post__by_member(self):
         """DELETE /assets/<id>.json (member)
@@ -627,7 +724,7 @@ class TestTypePad(unittest.TestCase):
         self.assertEquals(comment.in_reply_to.url_id, asset.url_id)
         self.assertEquals(comment.content, content)
 
-        self.testdata['comments_created'].append(comment.xid)
+        self.testdata['comments_created'].append(comment.url_id)
 
     @attr(user='group')
     def test_1_GET_assets_id_favorites(self):
@@ -686,8 +783,8 @@ class TestTypePad(unittest.TestCase):
 
         asset = typepad.Asset.from_dict(data)
 
-        object_type = asset.primary_object_type() or asset.object_type
-        post_type = object_type.split(':')[2].lower()
+        object_type = asset.object_type
+        post_type = object_type.lower()
 
         asset.groups = [ group.id ]
         response, content = typepad.api.browser_upload.upload(
@@ -713,9 +810,8 @@ class TestTypePad(unittest.TestCase):
             posted_asset = typepad.Asset.get(parts[2], batch=False)
 
             self.assertValidAsset(posted_asset)
-            self.assertEquals(posted_asset.primary_object_type(), data['objectTypes'][0])
 
-            self.testdata['assets_created'].append(posted_asset.xid)
+            self.testdata['assets_created'].append(posted_asset.url_id)
         else: # test for a failure
             self.assertEquals(int(params['status'][0]), httplib.FORBIDDEN)
 
@@ -726,7 +822,7 @@ class TestTypePad(unittest.TestCase):
 
         asset = {
             'content': 'This is a test upload',
-            'objectTypes': ['tag:api.typepad.com,2009:Photo']
+            'objectType': 'Photo',
         }
         content = "\n".join(
             """iVBORw0KGgoAAAANSUhEUgAAACAAAAAWCAYAAAChWZ5EAAAAqUlEQVRIx8XTsQ2AIBCFYcM+OoOV"""
@@ -743,7 +839,7 @@ class TestTypePad(unittest.TestCase):
 
         asset = {
             'content': 'A small audio post',
-            'objectTypes': ['tag:api.typepad.com,2009:Audio']
+            'objectType': 'Audio',
         }
         content = "\n".join(
             """//tQZAAAAAAAf4UAAAgAAA0goAABFVWnQ7kKAAAAADSDAAAAgEAYCAQCAUBAIAYABADkguG8AkDk"""
@@ -779,7 +875,7 @@ class TestTypePad(unittest.TestCase):
 
         asset = {
             'content': 'This is a test upload',
-            'objectTypes': ['tag:api.typepad.com,2009:Photo']
+            'objectType': 'Photo',
         }
         content = "\n".join(
             """iVBORw0KGgoAAAANSUhEUgAAACAAAAAWCAYAAAChWZ5EAAAAqUlEQVRIx8XTsQ2AIBCFYcM+OoOV"""
@@ -819,6 +915,28 @@ class TestTypePad(unittest.TestCase):
         self.assertEquals(fav.in_reply_to.url_id, asset_id)
         self.assertEquals(fav.author.url_id, member_id)
 
+    @attr(user='member')
+    def test_6z_HEAD_favorites_id(self):
+        """HEAD /favorites/<asset_id>:<user_id>.json (group)
+        
+        Tests existence of a favorite using group credentials.
+        """
+        # '6z' will sort this test so it runs prior to the favorite deletion
+        # tests.
+        self.assert_(len(self.testdata['assets_created']) > 0)
+
+        asset_id = self.testdata['assets_created'][0]
+        member_id = self.testdata['member']['xid']
+        admin_id = self.testdata['admin']['xid']
+
+        typepad.client.batch_request()
+        fav = typepad.Favorite.head_by_user_asset(member_id, asset_id)
+        not_fav = typepad.Favorite.head_by_user_asset(admin_id, asset_id)
+        typepad.client.complete_batch()
+
+        self.assert_(fav.found(), "asset is not a favorite by member but should be")
+        self.assert_(not not_fav.found(), "asset is a favorite of admin but shouldn't be")
+
     @attr(user='group')
     def test_7_DELETE_favorites_id__by_group(self):
         """DELETE /favorites/<id>.json (group)
@@ -852,27 +970,6 @@ class TestTypePad(unittest.TestCase):
         fav = typepad.Favorite.get_by_user_asset(member_id, asset_id)
         self.assertNotFound(typepad.client.complete_batch)
 
-    @attr(user='admin')
-    def test_7_DELETE_favorites_id__by_admin(self):
-        """DELETE /favorites/<id>.json (admin)
-        
-        Tests deletion of a favorite object using admin credentials.
-        """
-
-        raise nose.SkipTest(
-            'FIXME: https://intranet.sixapart.com/bugs/default.asp?87902')
-
-        self.assert_(len(self.testdata['assets_created']))
-
-        member_id = self.testdata['member']['xid']
-        asset_id = self.testdata['assets_created'][1]
-
-        typepad.Favorite.get_by_user_asset(member_id, asset_id).delete()
-
-        typepad.client.batch_request()
-        fav = typepad.Favorite.get_by_user_asset(member_id, asset_id)
-        self.assertNotFound(typepad.client.complete_batch)
-
     @attr(user='group')
     def test_1_GET_groups_id(self):
         """GET /groups/<id>.json (group)
@@ -938,7 +1035,7 @@ class TestTypePad(unittest.TestCase):
         typepad.Group.get_by_url_id(group_id).link_assets.post(link)
         self.assertValidAsset(link)
 
-        self.testdata['assets_created'].append(link.xid)
+        self.testdata['assets_created'].append(link.url_id)
 
     @attr(user='group')
     def test_1_GET_groups_id_memberships(self):
@@ -961,9 +1058,9 @@ class TestTypePad(unittest.TestCase):
             self.assertValidRelationship(rel)
 
         self.assert_(len(everyone) > 0, 'memberships should be non-zero')
-        self.assert_(admin_id in [a.target.xid for a in everyone],
+        self.assert_(admin_id in [a.target.url_id for a in everyone],
             'configured admin should exist in memberships')
-        self.assert_(member_id in [a.target.xid for a in everyone],
+        self.assert_(member_id in [a.target.url_id for a in everyone],
             'configured member should exist in memberships')
 
     @attr(user='group')
@@ -990,7 +1087,7 @@ class TestTypePad(unittest.TestCase):
 
         # yes, we have one or more admins!
         self.assert_(len(admins) > 0, 'admin membership should be non-zero')
-        self.assert_(admin_id in [a.target.xid for a in admins],
+        self.assert_(admin_id in [a.target.url_id for a in admins],
             'configured admin should exist in admin memberships')
 
         self.assertNotEqual(admins.total_results, everyone.total_results,
@@ -1019,7 +1116,7 @@ class TestTypePad(unittest.TestCase):
         for member in members:
             self.assertValidRelationship(member)
 
-        self.assert_(member_id in [a.target.xid for a in members],
+        self.assert_(member_id in [a.target.url_id for a in members],
             'configured admin should exist in admin memberships')
 
     @attr(user='group')
@@ -1064,7 +1161,7 @@ class TestTypePad(unittest.TestCase):
         typepad.Group.get_by_url_id(group_id).post_assets.post(post)
         self.assertValidAsset(post)
 
-        self.testdata['assets_created'].append(post.xid)
+        self.testdata['assets_created'].append(post.url_id)
 
     @attr(user='group')
     def test_5_POST_groups_id_post_assets__by_group(self):
@@ -1078,7 +1175,7 @@ class TestTypePad(unittest.TestCase):
         post = self.post_asset()
         post.content = 'Test post asset by group'
 
-        self.assertUnauthorized(
+        self.assertForbidden(
             typepad.Group.get_by_url_id(group_id).post_assets.post, post)
 
     @attr(user='blocked')
@@ -1110,7 +1207,7 @@ class TestTypePad(unittest.TestCase):
         typepad.Group.get_by_url_id(group_id).video_assets.post(video)
         self.assertValidAsset(video)
 
-        self.testdata['assets_created'].append(video.xid)
+        self.testdata['assets_created'].append(video.url_id)
 
     @attr(user='group')
     def test_5_POST_groups_id_video_assets__by_group(self):
@@ -1124,7 +1221,7 @@ class TestTypePad(unittest.TestCase):
         video = self.video_asset()
         video.content = 'Test video post by group'
 
-        self.assertUnauthorized(
+        self.assertForbidden(
             typepad.Group.get_by_url_id(group_id).video_assets.post, video)
 
     @attr(user='blocked')
@@ -1215,7 +1312,7 @@ class TestTypePad(unittest.TestCase):
         self.assertValidUser(user)
         self.assert_(len(memberships) == 1)
         self.assertValidRelationship(memberships[0])
-        self.assertEquals(memberships[0].source.xid, group_id)
+        self.assertEquals(memberships[0].source.url_id, group_id)
 
         if action == 'block':
             self.assert_(not memberships[0].is_blocked())
@@ -1329,16 +1426,7 @@ class TestTypePad(unittest.TestCase):
         typepad.client.complete_batch()
 
         self.assertValidUser(user)
-        self.assertEquals(user.xid, member_id)
-
-    @attr(user='group')
-    def test_1_GET_users_invalid(self):
-        """GET /users/(invalid).json (group)
-        """
-
-        typepad.client.batch_request()
-        self.assertRaises(ValueError, typepad.User.get_by_url_id, '(invalid)')
-        typepad.client.complete_batch()
+        self.assertEquals(user.url_id, member_id)
 
     @attr(user='group')
     def test_1_GET_users__(self):
@@ -1346,8 +1434,10 @@ class TestTypePad(unittest.TestCase):
         """
 
         typepad.client.batch_request()
-        self.assertRaises(ValueError, typepad.User.get_by_url_id, '')
-        typepad.client.complete_batch()
+        try:
+            self.assertRaises(ValueError, typepad.User.get_by_url_id, '')
+        finally:
+            typepad.client.complete_batch()
 
     @attr(user='group')
     def test_1_GET_users_id_elsewhere_accounts(self):
@@ -1400,8 +1490,7 @@ class TestTypePad(unittest.TestCase):
         self.assertValidUser(user)
         self.assertValidFilter(listset)
 
-        # FIXME: https://intranet.sixapart.com/bugs/default.asp?87911
-        events = [x for x in listset[0] if x.object]
+        events = listset[0]
         for event in events:
             self.assertValidEvent(event)
 
@@ -1409,28 +1498,6 @@ class TestTypePad(unittest.TestCase):
         self.assertEquals(events[0].object.author.url_id, member_id)
         self.assertTrue(events[0].object.groups[0].endswith(group_id))
 
-    @attr(user='group')
-    def test_1_GET_users_id_favorites(self):
-        """GET /users/<id>/favorites.json (group)
-        
-        Tests the selection of the favorites for a specific user.
-        """
-
-        raise nose.SkipTest(
-            'FIXME: https://intranet.sixapart.com/bugs/default.asp?87904')
-
-        member_id = self.testdata['member']['xid']
-
-        typepad.client.batch_request()
-        user = typepad.User.get_by_url_id(member_id)
-        listset = self.filterEndpoint(user.favorites)
-        typepad.client.complete_batch()
-
-        self.assertValidFilter(listset)
-        favs = listset[0]
-        for fav in favs:
-            self.assertValidFavorite(fav)
-
     @attr(user='member')
     def test_6_POST_users_id_favorites(self):
         """POST /users/<id>/favorites.json (member)
@@ -1465,6 +1532,28 @@ class TestTypePad(unittest.TestCase):
 
             self.assertValidFavorite(fav2)
 
+    @attr(user='group')
+    def test_6z_GET_users_id_favorites(self):
+        """GET /users/<id>/favorites.json (group)
+        
+        Tests the selection of the favorites for a specific user.
+        """
+
+        raise nose.SkipTest(
+            'FIXME: https://intranet.sixapart.com/bugs/default.asp?92991')
+
+        member_id = self.testdata['member']['xid']
+
+        typepad.client.batch_request()
+        user = typepad.User.get_by_url_id(member_id)
+        listset = self.filterEndpoint(user.favorites)
+        typepad.client.complete_batch()
+
+        self.assertValidFilter(listset)
+        favs = listset[0]
+        for fav in favs:
+            self.assertValidFavorite(fav)
+
     @attr(user='group')
     def test_1_GET_users_id_memberships(self):
         """GET /users/<id>/memberships.json (group)
@@ -1483,7 +1572,7 @@ class TestTypePad(unittest.TestCase):
         self.assertValidFilter(listset)
 
         memberships = listset[0]
-        self.assert_(group_id in [x.source.xid for x in memberships])
+        self.assert_(group_id in [x.source.url_id for x in memberships])
 
     @attr(user='group')
     def test_1_GET_users_id_memberships_admin(self):
@@ -1505,7 +1594,7 @@ class TestTypePad(unittest.TestCase):
         # self.assertValidFilter(listset)
 
         memberships = listset[0]
-        self.assert_(group_id in [x.source.xid for x in memberships])
+        self.assert_(group_id in [x.source.url_id for x in memberships])
 
     @attr(user='group')
     def test_1_GET_users_id_memberships_by_group_id(self):
@@ -1529,7 +1618,7 @@ class TestTypePad(unittest.TestCase):
         # self.assertValidFilter(listset)
 
         # memberships = listset[0]
-        self.assert_(member_id in [x.target.xid for x in memberships])
+        self.assert_(member_id in [x.target.url_id for x in memberships])
 
     @attr(user='group')
     def test_1_GET_users_id_memberships_member(self):
@@ -1552,7 +1641,7 @@ class TestTypePad(unittest.TestCase):
         # self.assertValidFilter(listset)
 
         # memberships = listset[0]
-        self.assert_(group_id in [x.source.xid for x in memberships])
+        self.assert_(group_id in [x.source.url_id for x in memberships])
 
     @attr(user='group')
     def test_1_GET_users_id_notifications(self):
@@ -1561,6 +1650,9 @@ class TestTypePad(unittest.TestCase):
         Tests the endpoint for gathering a user's notifications.
         """
 
+        raise nose.SkipTest(
+            'FIXME: https://intranet.sixapart.com/bugs/default.asp?95720')
+
         self.assert_(len(self.testdata['assets']) >= 2,
             'Must have 2 or more assets to test')
 
@@ -1579,8 +1671,7 @@ class TestTypePad(unittest.TestCase):
         for event in inbox:
             self.assertValidEvent(event)
 
-        # FIXME: https://intranet.sixapart.com/bugs/default.asp?88601
-        # self.assert_(asset_id in [x.object.url_id for x in inbox if x.object])
+        self.assert_(asset_id in [x.object.url_id for x in inbox])
 
     @attr(user='group')
     def test_1_GET_users_id_relationships(self):
@@ -1603,7 +1694,7 @@ class TestTypePad(unittest.TestCase):
         for contact in contacts:
             self.assertValidRelationship(contact)
 
-        self.assert_(member_id in [x.source.xid for x in contacts])
+        self.assert_(member_id in [x.source.url_id for x in contacts])
 
     @attr(user='group')
     def test_1_GET_users_id_relationships_by_group_id(self):
@@ -1628,7 +1719,7 @@ class TestTypePad(unittest.TestCase):
         for contact in contacts:
             self.assertValidRelationship(contact)
 
-        self.assert_(member_id in [x.source.xid for x in contacts])
+        self.assert_(member_id in [x.source.url_id for x in contacts])
 
     @attr(user='group')
     def test_1_GET_users_id_relationships_follower(self):
@@ -1652,8 +1743,8 @@ class TestTypePad(unittest.TestCase):
         for contact in contacts:
             self.assertValidRelationship(contact)
 
-        self.assert_(admin_id in [x.source.xid for x in contacts])
-        self.assert_(member_id in [x.target.xid for x in contacts])
+        self.assert_(admin_id in [x.source.url_id for x in contacts])
+        self.assert_(member_id in [x.target.url_id for x in contacts])
 
     @attr(user='group')
     def test_1_GET_users_id_relationships_follower_by_group_id(self):
@@ -1679,8 +1770,8 @@ class TestTypePad(unittest.TestCase):
         for contact in contacts:
             self.assertValidRelationship(contact)
 
-        self.assert_(admin_id in [x.source.xid for x in contacts])
-        self.assert_(member_id in [x.target.xid for x in contacts])
+        self.assert_(admin_id in [x.source.url_id for x in contacts])
+        self.assert_(member_id in [x.target.url_id for x in contacts])
 
     @attr(user='group')
     def test_1_GET_users_id_relationships_following(self):
@@ -1704,8 +1795,8 @@ class TestTypePad(unittest.TestCase):
         for contact in contacts:
             self.assertValidRelationship(contact)
 
-        self.assert_(member_id in [x.source.xid for x in contacts])
-        self.assert_(admin_id in [x.target.xid for x in contacts])
+        self.assert_(member_id in [x.source.url_id for x in contacts])
+        self.assert_(admin_id in [x.target.url_id for x in contacts])
 
     @attr(user='group')
     def test_1_GET_users_id_relationships_following_by_group_id(self):
@@ -1731,8 +1822,8 @@ class TestTypePad(unittest.TestCase):
         for contact in contacts:
             self.assertValidRelationship(contact)
 
-        self.assert_(member_id in [x.source.xid for x in contacts])
-        self.assert_(admin_id in [x.target.xid for x in contacts])
+        self.assert_(member_id in [x.source.url_id for x in contacts])
+        self.assert_(admin_id in [x.target.url_id for x in contacts])
 
     @attr(user='member')
     def test_1_GET_users_self__by_member(self):
@@ -1746,7 +1837,10 @@ class TestTypePad(unittest.TestCase):
         typepad.client.complete_batch()
 
         self.assertValidUser(user)
-        self.assertEquals(user.xid, member_id)
+        self.assertEquals(user.url_id, member_id)
+
+        scheme, netloc, path, query, fragment = urlsplit(user._location)
+        self.assertEquals(path, '/users/%s.json' % user.url_id)
 
     @attr(user='group')
     def test_1_GET_users_id__using_id(self):
@@ -1754,14 +1848,15 @@ class TestTypePad(unittest.TestCase):
         """
 
         member_id = self.testdata['member']['xid']
-        uri = "tag:api.typepad.com,2009:%s" % member_id
 
         typepad.client.batch_request()
-        user = typepad.User.get_by_id(uri)
-        typepad.client.complete_batch()
+        try:
+            user = typepad.User.get_by_url_id(member_id)
+        finally:
+            typepad.client.complete_batch()
 
         self.assertValidUser(user)
-        self.assertEquals(user.xid, member_id)
+        self.assertEquals(user.url_id, member_id)
 
     @attr(user='group')
     def test_1_GET_assets_id__using_id(self):
@@ -1769,14 +1864,15 @@ class TestTypePad(unittest.TestCase):
         """
 
         asset_id = self.testdata['assets'][1]
-        uri = "tag:api.typepad.com,2009:%s" % asset_id
 
         typepad.client.batch_request()
-        asset = typepad.Asset.get_by_id(uri)
-        typepad.client.complete_batch()
+        try:
+            asset = typepad.Asset.get_by_url_id(asset_id)
+        finally:
+            typepad.client.complete_batch()
 
         self.assertValidAsset(asset)
-        self.assertEquals(asset.xid, asset_id)
+        self.assertEquals(asset.url_id, asset_id)
 
     @attr(user='group')
     def test_1_GET_groups_id__using_id(self):
@@ -1784,14 +1880,15 @@ class TestTypePad(unittest.TestCase):
         """
 
         group_id = self.testdata['group']['xid']
-        uri = "tag:api.typepad.com,2009:%s" % group_id
 
         typepad.client.batch_request()
-        group = typepad.Group.get_by_id(uri)
-        typepad.client.complete_batch()
+        try:
+            group = typepad.Group.get_by_url_id(group_id)
+        finally:
+            typepad.client.complete_batch()
 
         self.assertValidGroup(group)
-        self.assertEquals(group.xid, group_id)
+        self.assertEquals(group.url_id, group_id)
 
     ### Supporting functions for this test suite
 
@@ -1830,7 +1927,7 @@ class TestTypePad(unittest.TestCase):
         for event in events:
             # FIXME: https://intranet.sixapart.com/bugs/default.asp?88008
             if event.object and len(event.object.groups) > 0:
-                self.testdata['assets'].append(event.object.xid)
+                self.testdata['assets'].append(event.object.url_id)
 
         self.clear_credentials()
 
@@ -1859,7 +1956,6 @@ class TestTypePad(unittest.TestCase):
         self.assert_(isinstance(asset, typepad.Asset),
             'object %r is not a typepad.Asset' % asset)
         self.assert_(asset.author)
-        self.assert_(asset.actor)
         self.assertValidUser(asset.author)
         # asset.content is not required for some asset types
         # self.assert_(asset.content)
@@ -1870,8 +1966,7 @@ class TestTypePad(unittest.TestCase):
         self.assert_(asset.id)
         self.assert_(asset.url_id)
         self.assert_(len(asset.object_types) > 0)
-        object_type = asset.primary_object_type()
-        self.assert_(object_type)
+        self.assert_(asset.object_type)
         self.assert_(asset.published)
 
         self.assert_(asset.make_self_link())
@@ -1880,28 +1975,28 @@ class TestTypePad(unittest.TestCase):
         # TODO: alternate was optional before, so maybe permalink_url might be None sometimes?
         self.assert_(asset.permalink_url is not None)
 
-        self.assert_(object_type.startswith('tag:api.typepad.com'))
+        object_type = asset.object_type
         self.assertValidAssetRef(asset.asset_ref)
-        if object_type == 'tag:api.typepad.com,2009:Link':
+        if object_type == 'Link':
             # additional properties we expect for link assets
             self.assert_(asset.target_url is not None)
-        elif object_type == 'tag:api.typepad.com,2009:Photo':
+        elif object_type == 'Photo':
             # additional properties we expect for photo assets
             self.assert_(asset.image_link is not None)
             self.assert_(asset.image_link.url_template is not None)
             self.assert_(asset.image_link.width is not None)
             self.assert_(asset.image_link.height is not None)
-        elif object_type == 'tag:api.typepad.com,2009:Audio':
+        elif object_type == 'Audio':
             self.assert_(asset.audio_link is not None)
             self.assert_(asset.audio_link.url is not None)
-        elif object_type == 'tag:api.typepad.com,2009:Post':
+        elif object_type == 'Post':
             pass
-        elif object_type == 'tag:api.typepad.com,2009:Comment':
+        elif object_type == 'Comment':
             # FIXME: we have a data problem for one of our comments; parent object was deleted but the comment remained
             if len(asset.groups) > 0:
                 self.assert_(asset.in_reply_to)
                 self.assertValidAssetRef(asset.in_reply_to)
-        elif object_type == 'tag:api.typepad.com,2009:Video':
+        elif object_type == 'Video':
             self.assert_(asset.video_link is not None)
             self.assert_(asset.video_link.embed_code is not None)
 
@@ -1917,9 +2012,9 @@ class TestTypePad(unittest.TestCase):
             self.assert_(asset.source.permalink_url)
             self.assert_(asset.source.provider)
 
-            self.assert_(asset.source.provider.icon)
-            self.assert_(asset.source.provider.name)
-            self.assert_(asset.source.provider.uri)
+            self.assert_(asset.source.provider['icon'])
+            self.assert_(asset.source.provider['name'])
+            self.assert_(asset.source.provider['uri'])
 
     def assertValidUser(self, user):
         """Checks given asset for properties that should be present on all assets.
@@ -1932,6 +2027,7 @@ class TestTypePad(unittest.TestCase):
         self.assert_(len(user.object_types) > 0)
         self.assertEquals(user.object_types[0],
             'tag:api.typepad.com,2009:User')
+        self.assertEquals(user.object_type, 'User')
         # this asserts as false when the user's interests is empty
         # self.assert_(user.interests)
 
@@ -1951,7 +2047,6 @@ class TestTypePad(unittest.TestCase):
         self.assert_(isinstance(event, typepad.Event),
             'object %r is not a typepad.Event' % event)
         self.assert_(event.id)
-        self.assert_(event.xid)
         self.assert_(event.url_id)
         self.assert_(event.published)
         self.assert_(event.make_self_link())
@@ -1987,6 +2082,7 @@ class TestTypePad(unittest.TestCase):
         self.assert_(len(group.object_types) > 0)
         self.assertEquals(group.object_types[0],
             'tag:api.typepad.com,2009:Group')
+        self.assertEquals(group.object_type, 'Group')
 
     def assertValidApplication(self, app):
         self.assert_(isinstance(app, typepad.Application),
@@ -2038,33 +2134,33 @@ class TestTypePad(unittest.TestCase):
             self.assert_(len(rel.status.types) > 0)
             self.assert_(rel.status.types[0] in rel.created)
         self.assert_(rel.source)
-        if rel.source.object_types[0] == 'tag:api.typepad.com,2009:Group':
+        if rel.source.object_type == 'Group':
             self.assertValidGroup(rel.source)
-        elif rel.source.object_types[0] == 'tag:api.typepad.com,2009:User':
+        elif rel.source.object_type == 'User':
             self.assertValidUser(rel.source)
         else:
             self.fail('unexpected object type %s for relationship source' % \
                 rel.source.object_types[0])
         self.assert_(rel.target)
-        if rel.target.object_types[0] == 'tag:api.typepad.com,2009:Group':
+        if rel.target.object_type == 'Group':
             self.assertValidGroup(rel.target)
-        elif rel.target.object_types[0] == 'tag:api.typepad.com,2009:User':
+        elif rel.target.object_type == 'User':
             self.assertValidUser(rel.target)
         else:
             self.fail('unexpected object type %s for relationship target' % \
-                rel.target.object_types[0])
+                rel.target.object_type)
 
     def assertValidAssetRef(self, ref):
         self.assert_(ref.url_id)
         self.assert_(isinstance(ref, typepad.AssetRef),
             'object %r is not a typepad.AssetRef' % ref)
-        # FIXME: https://intranet.sixapart.com/bugs/default.asp?87953
-        # self.assert_(ref.href)
-        # self.assertEquals(ref.type, 'application/json')
+        self.assert_(ref.href)
+        self.assertEquals(ref.type, 'application/json')
         self.assertValidUser(ref.author)
         self.assert_(ref.object_types)
         self.assert_(len(ref.object_types) > 0)
         self.assert_(ref.object_types[0].startswith('tag:api.typepad.com'))
+        self.assert_(ref.object_type)
 
     def filterEndpoint(self, endpoint, *args, **kwargs):
         full = endpoint.filter(**kwargs)
@@ -2076,8 +2172,8 @@ class TestTypePad(unittest.TestCase):
 
     def assertValidFilter(self, listset):
         (full, slice1, slice2, slice3, slice4) = listset
-        self.assert_(isinstance(full, typepad.ListObject),
-            'object %r is not a typepad.ListObject' % full)
+        self.assert_(isinstance(full, typepad.ListObject) or isinstance(full, typepad.StreamObject),
+            'object %r is not a typepad.ListObject or typepad.StreamObject' % full)
 
         self.assert_(len(full.entries) <= 50, 'API result exceeded 50 entries')
 
@@ -2121,7 +2217,7 @@ class TestTypePad(unittest.TestCase):
 
     def link_asset(self):
         """Creates a Link asset instance for testing purposes."""
-        link = typepad.LinkAsset()
+        link = typepad.Link()
         link.target_url = 'http://www.typepad.com/'
         link.title = ''
         link.content = 'Test link post'
